/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.world.classic.worldgen.trees;

import java.util.Random;

import net.minecraft.block.BlockLog;
import net.minecraft.block.properties.PropertyBool;
import net.minecraft.block.state.IBlockState;
import net.minecraft.init.Blocks;
import net.minecraft.util.ResourceLocation;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;
import net.minecraft.world.gen.structure.template.PlacementSettings;
import net.minecraft.world.gen.structure.template.Template;
import net.minecraft.world.gen.structure.template.TemplateManager;

import net.dries007.tfc.Constants;
import net.dries007.tfc.TerraFirmaCraft;
import net.dries007.tfc.api.ITreeGenerator;
import net.dries007.tfc.api.types.Tree;
import net.dries007.tfc.objects.blocks.BlocksTFC;
import net.dries007.tfc.objects.blocks.wood.BlockLeavesTFC;
import net.dries007.tfc.objects.blocks.wood.BlockLogTFC;

import static net.dries007.tfc.objects.blocks.wood.BlockLogTFC.PLACED;
import static net.minecraft.block.BlockLog.LOG_AXIS;
import static net.minecraft.block.BlockVine.*;

public class TreeGenKapok implements ITreeGenerator
{
    private IBlockState trunk;
    private static final PlacementSettings settings = ITreeGenerator.getDefaultSettings();
    private IBlockState bark;

    private static final BlockPos[] trunkPos = new BlockPos[] {
        new BlockPos(0, 0, 0), new BlockPos(-1, 0, 0), new BlockPos(0, 0, -1), new BlockPos(-1, 0, -1)
    };

    @Override
    public void generateTree(TemplateManager manager, World world, BlockPos pos, Tree tree, Random rand)
    {
        trunk = BlockLogTFC.get(tree).func_176223_P().func_177226_a(PLACED, false);
        bark = BlockLogTFC.get(tree).func_176223_P().func_177226_a(PLACED, false).func_177226_a(field_176299_a, BlockLog.EnumAxis.NONE);

        int height = 12 + rand.nextInt(8);
        int branches = 2 + rand.nextInt(3);

        int x1, y1, z1, type;
        for (int i = 0; i < branches; i++)
        {
            y1 = 6 + rand.nextInt(height - 8);
            x1 = rand.nextInt(3);
            z1 = rand.nextInt(3);
            if (rand.nextBoolean())
                x1 = -x1 - 1;
            if (rand.nextBoolean())
                x1 = -z1 - 1;
            if (x1 == 0 || x1 == -1)
                x1 = x1 * 3 + 1;
            if (z1 == 0 || z1 == -1)
                z1 = z1 * 3 + 1;
            type = 1 + rand.nextInt(3);
            placeBranch(manager, world, pos.func_177982_a(x1, y1, z1), tree.name() + "/branch" + type);
            checkAndPlace(world, pos.func_177982_a(x1 - Math.abs(x1) / x1, y1 - 1, z1 - Math.abs(z1) / z1), true);
        }

        for (int i = -1; i < height; i++)
            placeTrunk(world, pos.func_177982_a(0, i, 0));
        placeBranch(manager, world, pos.func_177982_a(0, height, 0), tree.name() + "/top");

    }

    @Override
    public boolean canGenerateTree(World world, BlockPos pos, Tree treeType)
    {
        for (BlockPos p1 : trunkPos)
        {
            if (BlocksTFC.isSoil(world.func_180495_p(pos.func_177971_a(p1))))
                continue;
            if (world.func_180495_p(pos.func_177971_a(p1)).func_185904_a().func_76222_j())
            {
                if (BlocksTFC.isSoil(world.func_180495_p(pos.func_177971_a(p1).func_177979_c(1))))
                    continue;
                if (BlocksTFC.isSoil(world.func_180495_p(pos.func_177971_a(p1).func_177979_c(2))) && world.func_180495_p(pos.func_177971_a(p1.func_177979_c(1))).func_185904_a().func_76222_j())
                    continue;
            }
            return false;
        }

        return ITreeGenerator.super.canGenerateTree(world, pos, treeType);
    }

    private void placeBranch(TemplateManager manager, World world, BlockPos pos, String name)
    {
        ResourceLocation base = new ResourceLocation(Constants.MOD_ID, name);
        Template structureBase = manager.func_189942_b(world.func_73046_m(), base);

        if (structureBase == null)
        {
            TerraFirmaCraft.getLog().warn("Unable to find a template for " + base.toString());
            return;
        }
        BlockPos size = structureBase.func_186259_a();
        pos = pos.func_177982_a(-size.func_177958_n() / 2, 0, -size.func_177952_p() / 2);

        ITreeGenerator.addStructureToWorld(world, pos, structureBase, settings);
    }

    private void placeTrunk(World world, BlockPos pos)
    {
        for (BlockPos p1 : trunkPos)
            checkAndPlace(world, pos.func_177971_a(p1), false);

        placeVine(world, pos.func_177982_a(1, 0, 0), field_176280_O);
        placeVine(world, pos.func_177982_a(1, 0, -1), field_176280_O);
        placeVine(world, pos.func_177982_a(0, 0, 1), field_176273_b);
        placeVine(world, pos.func_177982_a(-1, 0, 1), field_176273_b);
        placeVine(world, pos.func_177982_a(-2, 0, 0), field_176278_M);
        placeVine(world, pos.func_177982_a(-2, 0, -1), field_176278_M);
        placeVine(world, pos.func_177982_a(-1, 0, -2), field_176279_N);
        placeVine(world, pos.func_177982_a(0, 0, -2), field_176279_N);
    }

    private void checkAndPlace(World world, BlockPos pos, boolean useBark)
    {
        if (world.func_180495_p(pos).func_185904_a().func_76222_j() || world.func_180495_p(pos).func_177230_c() instanceof BlockLeavesTFC)
            world.func_175656_a(pos, useBark ? bark : trunk);
    }

    private void placeVine(World world, BlockPos pos, PropertyBool prop)
    {
        if (Math.random() < 0.8f && world.func_180495_p(pos).func_177230_c() == Blocks.field_150350_a)
            world.func_175656_a(pos, Blocks.field_150395_bd.func_176223_P().func_177226_a(prop, true));

    }
}
